@@ -101,17 +101,29 @@ class AcceptSendingDataEvent(ContractEvent):
 
 
 @dataclass
-class NoticeTaskCompletionEvent(ContractEvent):
-    """A event of noticing task manager that current task is complete."""
+class UploadTaskAchievementEvent(ContractEvent):
+    """A event of uploading task achievement materials."""
 
-    TYPE = 'notify_task_completion'
+    TYPE = 'upload_task_achieve'
 
-    result: bool
     report: str
     model: str
 
     @classmethod
-    def contract_to_event(cls, contract: dict) -> 'AcceptSendingDataEvent':
+    def contract_to_event(cls, contract: dict) -> None:
+        raise NotImplementedError()
+
+
+@dataclass
+class NoticeTaskCompletionEvent(ContractEvent):
+    """A event of noticing task manager that current task is complete."""
+
+    TYPE = 'notice_task_completion'
+
+    result: bool
+
+    @classmethod
+    def contract_to_event(cls, contract: dict) -> None:
         raise NotImplementedError()
 
 
@@ -142,6 +154,7 @@ class TaskMessageContractor(Contractor):
     _URL = 'http://federated-service:9080/fed-service/api/v1/message'
 
     _TASK_INSIDE = 'TaskInside'
+    _UPLOAD_TASK_ACHIEVE = 'TaskResultUpload'
     _TASK_FINISH = 'TaskFinish'
 
     def __init__(self, task_id: str) -> None:
@@ -282,40 +295,49 @@ class TaskMessageContractor(Contractor):
         """Query address of the target. TODO refactory contractors."""
         return self._task_contractor.query_address(target=target)
 
-    def notify_task_completion(self,
+    def upload_task_achivement(self,
                                aggregator: str,
-                               result: bool,
-                               report_file: str,
-                               model_file: str):
-        """Notify task manager a task is complete.
+                               model_file: str,
+                               report_file: str = None):
+        """Upload achivement materials after a task is complete.
 
         :args
             :aggregator
                 The ID of the aggregator.
-            :result
-                True if successful otherwize False.
-            :report_file
-                Local path of the report file.
             :model_file
                 Local path of the model file.
+            :report_file
+                Local path of the report file.
         """
         assert aggregator and isinstance(aggregator, str), f'unknown aggregator: {aggregator}'
-        assert result is not None and isinstance(result, bool), 'result must be True or False'
-        assert (
-            report_file and isinstance(report_file, str)
-            and os.path.exists(report_file)
-            and os.path.isfile(report_file)
-        ), f'the report_file does not exist or can not be accessed: {report_file}'
         assert (
             model_file and isinstance(model_file, str)
             and os.path.exists(model_file)
             and os.path.isfile(model_file)
         ), f'the model_file does not exist or can not be accessed: {model_file}'
+        if report_file:
+            assert (
+                isinstance(report_file, str)
+                and os.path.exists(report_file)
+                and os.path.isfile(report_file)
+            ), f'the report_file does not exist or can not be accessed: {report_file}'
 
-        event = NoticeTaskCompletionEvent(result=result,
-                                          report=report_file,
-                                          model=model_file)
+        event = UploadTaskAchievementEvent(report=report_file, model=model_file)
         TxId = self._new_contract(targets=[aggregator],
+                                  event=event,
+                                  message_title=self._UPLOAD_TASK_ACHIEVE)
+        if not TxId:
+            raise ContractException('failed to notify task completion')
+
+    def notify_task_completion(self, result: bool):
+        """Notify task manager a task is complete.
+
+        :args
+            :result
+                True if successful otherwize False.
+        """
+        event = NoticeTaskCompletionEvent(result=result)
+        TxId = self._new_contract(targets=self.EVERYONE,
                                   event=event,
                                   message_title=self._TASK_FINISH)
         if not TxId:
