@@ -8,8 +8,7 @@ import sys
 from abc import ABCMeta, abstractmethod
 from collections import deque
 from dataclasses import dataclass
-from functools import wraps
-from typing import Deque, Dict, Iterable, List, Tuple
+from typing import Deque, Dict, List, Tuple
 from zipfile import ZipFile
 
 import torch
@@ -19,9 +18,8 @@ from torch.utils.data import DataLoader
 
 from .. import logger
 from ..data_channel import GRPCDataChannel, SendingError
-from ..metrics_trace import MetricsTrace
 from ..perf_bench import SimplePerfBench, SimplePerfBenchResult
-from ..scheduler import Scheduler
+from ..scheduler import ConfigError, Scheduler, TaskComplete
 from .contractor import (BenchmarkDoneEvent, CheckinEvent,
                          CheckinResponseEvent, CloseAggregatorElectionEvent,
                          CloseRoundEvent, DistributeParametersEvent,
@@ -32,18 +30,11 @@ from .contractor import (BenchmarkDoneEvent, CheckinEvent,
                          SyncStateResponseEvent, UploadTrainingResultsEvent)
 
 __all__ = [
-    'ConfigError',
-    'AggregationError',
     'FedAvgScheduler',
-    'FedSGDScheduler',
-    'register_metrics'
+    'FedSGDScheduler'
 ]
 
 
-class ConfigError(Exception):
-    ...
-
-
 class AggregationError(Exception):
     ...
 
@@ -52,10 +43,6 @@ class SkipRound(Exception):
     ...
 
 
-class _TaskComplete(Exception):
-    ...
-
-
 @dataclass
 class AggregatorSyncData:
 
@@ -183,11 +170,10 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
         self._latest_aggregators: Deque[str] = deque()
         self._round = 0
 
-        self._metrics_bucket: Dict[str, MetricsTrace] = {}
+        self._example_inputs = None
 
         self.model = None
         self.optimizer = None
-        self.example_inputs = None
 
     def _validate_config(self):
         if self.min_clients > self.max_clients:
@@ -264,7 +250,7 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
 
     @abstractmethod
     def test(self):
-        """Define the training steps.
+        """Define the testing steps.
 
         If you do not want to do testing after training, simply make it pass.
         """
@@ -302,8 +288,8 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
         self.validate_context()
 
         def save_example_inputs(module, input, output):
-            if self.example_inputs is None:
-                self.example_inputs = input
+            if self._example_inputs is None:
+                self._example_inputs = input
         self.model.register_forward_hook(save_example_inputs)
 
     def push_log(self, message: str):
@@ -311,9 +297,6 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
         super().push_log(message=message)
         logger.info(message)
 
-    def get_metrics(self, name: str) -> MetricsTrace:
-        return self._metrics_bucket.get(name)
-
     def _launch_process(self):
         try:
             assert self.status == self._INIT, 'must begin from initial status'
@@ -341,7 +324,7 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
                     self._switch_status(self._FINISHING)
                     self._close_task()
 
-        except _TaskComplete:
+        except TaskComplete:
             logger.info('training task complete')
 
     def _checkin(self):
@@ -366,7 +349,7 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
     def _sync_state(self):
         """Synchronize state before each round, so it's easier to manage the process.
 
-        As an aggregator, iterates round and broadcasts, resets context of the new round.
+        As an aggregator, iterates round, broadcasts and resets context of the new round.
         As a participant, synchronizes state and gives a response.
         """
         self.push_log('Synchronizing round state ...')
@@ -433,7 +416,11 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
         if self.is_aggregator:
             self._switch_status(self._FINISHING)
             self.contractor.finish_task()
-            self._upload_task_output()
+            report_file_path, model_file_path = self._prepare_task_output()
+            self.contractor.upload_task_achivement(aggregator=self.id,
+                                                   report_file=report_file_path,
+                                                   model_file=model_file_path)
+            self.contractor.notify_task_completion(result=True)
         self.push_log(f'Task {self.task_id} closed. Byebye!')
 
     def _wait_for_gathering(self):
@@ -496,7 +483,7 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
                 self._round = _event.round
                 return True
             elif isinstance(_event, FinishTaskEvent):
-                raise _TaskComplete()
+                raise TaskComplete()
         return False
 
     def _wait_for_sync_state(self, timeout: int = 0) -> bool:
@@ -514,7 +501,7 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
                 self.push_log('Successfully synchronized state with the aggregator.')
                 return
             elif isinstance(_event, FinishTaskEvent):
-                raise _TaskComplete()
+                raise TaskComplete()
 
     def _wait_for_benchmark(self) -> str:
         """Wait for someone complete benchmark task.
@@ -574,7 +561,7 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
                 self.contractor.report_benchmark_done(round=self._round, peer_id=self.id)
                 return
             elif isinstance(_event, FinishTaskEvent):
-                raise _TaskComplete()
+                raise TaskComplete()
 
     def _wait_for_election_result(self):
         logger.debug('_wait_for_election_result ...')
@@ -601,7 +588,7 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
                     return
 
             elif isinstance(_event, FinishTaskEvent):
-                raise _TaskComplete()
+                raise TaskComplete()
 
     def _run_as_aggregator(self):
         self._start_round()
@@ -645,10 +632,10 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
                                                   data_stream=buffer.getvalue(),
                                                   connection_timeout=connection_timeout,
                                                   timeout=timeout)
+                    self.push_log(f'Successfully distributed parameters to ID: {_target}')
                 except SendingError:
                     self.push_log(f'Failed to distribute parameters to ID: {_target}')
                     logger.exception(f'failed to send parameters to {_target}')
-                self.push_log(f'Successfully distributed parameters to ID: {_target}')
 
             results[_target] = True
 
@@ -693,8 +680,8 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
             or self._round == self.max_rounds
         ):
             self.push_log('Begin to make a model test.')
-            test_result = self.test()
-            self.push_log(f'The latest test results are: {test_result}')
+            self.test()
+            self.push_log('Finished a round of test.')
 
     def _wait_for_calculation(self) -> Tuple[Dict[str, torch.Tensor], int]:
         """Wait for every calculator finish its task or timeout."""
@@ -756,8 +743,8 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
             torch.save(self.state_dict(), f)
         self.push_log('Saved latest parameters locally.')
 
-    def _upload_task_output(self) -> Tuple[str, str]:
-        """Upload final output of the task to task manager after training.
+    def _prepare_task_output(self) -> Tuple[str, str]:
+        """Generate final output files of the task.
 
         :return
             Local paths of the report file and model file.
@@ -768,22 +755,20 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
             _file = f'{os.path.join(self._save_path, _name)}.csv'
             _metrics.to_csv(_file)
             metrics_files.append(_file)
-        report_file = f'{os.path.join(self._save_path, "report.zip")}'
+        report_file = os.path.join(self._save_path, "report.zip")
         with ZipFile(report_file, 'w') as report_zip:
             for _file in metrics_files:
                 report_zip.write(_file)
         report_file_path = os.path.abspath(report_file)
 
         # torch.jit doesn't work with a TemporaryFile
-        model_file = f'{os.path.join(self._save_path, "scripted.pt")}'
-        model_scripted = torch.jit.trace(self.model, self.example_inputs)
+        model_file = os.path.join(self._save_path, "scripted.pt")
+        model_scripted = torch.jit.trace(self.model, self._example_inputs)
         model_scripted.save(model_file)
         model_file_path = os.path.abspath(model_file)
 
-        self.contractor.notify_task_completion(aggregator=self.id,
-                                               result=True,
-                                               report_file=report_file_path,
-                                               model_file=model_file_path)
+        self.push_log('Task achievement files are ready.')
+        return report_file_path, model_file_path
 
     def _run_as_data_owner(self):
         try:
@@ -831,7 +816,7 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
                 self.push_log(f'Training of round {self._round} begins.')
                 return
             elif isinstance(_event, FinishTaskEvent):
-                raise _TaskComplete()
+                raise TaskComplete()
 
     def _wait_for_updating_model(self):
         """Wait for receiving latest parameters from aggregator."""
@@ -847,7 +832,7 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
                 self.push_log('Successfully received latest parameters.')
                 return
             elif isinstance(_event, FinishTaskEvent):
-                raise _TaskComplete()
+                raise TaskComplete()
 
     def _wait_for_uploading_model(self):
         """Wait for uploading trained parameters to aggregator."""
@@ -858,7 +843,7 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
                     continue
                 return
             elif isinstance(_event, FinishTaskEvent):
-                raise _TaskComplete()
+                raise TaskComplete()
 
     def _wait_for_closing_round(self):
         """Wait for closing current round of training."""
@@ -869,42 +854,7 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
                     continue
                 return
             elif isinstance(_event, FinishTaskEvent):
-                raise _TaskComplete()
-
-
-def register_metrics(name: str, keys: Iterable[str]):
-    """Register a metrics trace.
-
-    The trace fo the metrics during training will be included in the task's
-    running result and can be downloaded. However, the trace content should
-    be appended manually.
-
-    If the specified name has already registered before, nothing changes.
-
-    :args
-        :name
-            The name of the metrics trace.
-        :keys
-            The names of the metrics items.
-    """
-    def decorate(func):
-        if not name or not isinstance(name, str):
-            raise ValueError(f'invalid name of the metrics: {name}')
-        if not keys or not isinstance(keys, Iterable):
-            raise ValueError(f'invalid keys of the metrics: {keys}')
-
-        @wraps(func)
-        def wrapper(_self, *args, **kwargs):
-            if not isinstance(_self, FedAvgScheduler) or func.__name__ != 'test':
-                raise RuntimeError(
-                    'Only applicable on "test" function of a "FedAvgScheduler" object.'
-                )
-            if name not in _self._metrics_bucket:
-                _self._metrics_bucket[name] = MetricsTrace(headers=keys)
-            return func(_self, *args, **kwargs)
-
-        return wrapper
-    return decorate
+                raise TaskComplete()
 
 
 class FedSGDScheduler(FedAvgScheduler):
