@@ -16,13 +16,13 @@ import torch
 from numpy.random import Generator
 
 from .. import logger
+from ..scheduler import ConfigError, TaskComplete
 from ..secure import aes
 from ..secure.ecdhe import new_key_pair, new_shared_key
 from ..secure.shamir import SecretShare
-from . import AggregationError, FedAvgScheduler
 from .contractor import (CheckinEvent, FinishTaskEvent,
                          UploadTrainingResultsEvent)
-from .fed_avg import ConfigError, _TaskComplete
+from .fed_avg import AggregationError, FedAvgScheduler
 from .secure_contractor import (AdvertiseKeysEvent, AdvertiseKeysResponseEvent,
                                 DistributeUser1ListEvent,
                                 DistributeUser2ListEvent,
@@ -641,7 +641,7 @@ class SecureFedAvgScheduler(FedAvgScheduler):
                 self.push_log(f'ID: {self.id} has advertised keys.')
                 return
             elif isinstance(_event, FinishTaskEvent):
-                raise _TaskComplete()
+                raise TaskComplete()
 
     def _wait_for_user_1_list(self):
         """Wait for receiving the list of user_1."""
@@ -669,7 +669,7 @@ class SecureFedAvgScheduler(FedAvgScheduler):
                 return
 
             elif isinstance(_event, FinishTaskEvent):
-                raise _TaskComplete()
+                raise TaskComplete()
 
     def _generate_shared_data(self) -> List[str]:
         """Generate information for sharing depending on user_1 list."""
@@ -719,7 +719,7 @@ class SecureFedAvgScheduler(FedAvgScheduler):
                 return
 
             elif isinstance(_event, FinishTaskEvent):
-                raise _TaskComplete()
+                raise TaskComplete()
 
     def _mask_state_dict(self, state_dict: Dict[str, torch.Tensor]) -> Dict[str, torch.Tensor]:
         assert (
@@ -792,4 +792,4 @@ class SecureFedAvgScheduler(FedAvgScheduler):
                 return
 
             elif isinstance(_event, FinishTaskEvent):
-                raise _TaskComplete()
+                raise TaskComplete()
